;		  K1008 (C) MTU, typed in by Hans Otten DEC 2024 TASM32 syntax
;         .PAGE  'EQUATES AND DATA STORAGE'                                   
;         TEST AND EXERCISE PROGRAM FOR THE K-1008 VISABLE MEMORY. THIS       
;         PROGRAM IMPLEMENTS TWO TESTS OF THE K-1008 DISPLAY BOARD.           
;            THE FIRST TEST PERFORMS A GROSS CHECK OF MEMORY FUNCTION         
;         AND DEMONSTRATES THE ACCURACY OR LACK THEREOF OF THE DISPLAY        
;         GENERATOR CIRCUITS. THE PATTERNS GENERATED ARE CKECKERBOARDS        
;         OF VARIOUS SIZES. THE DISPLAY MONITOR SHOULD MAINTAIN STABLE        
;         SYNC EVEN WITH THE LARGE WHITE AND DARK AREAS THAT MIGHT            
;         APPEAR. THE DIMENSIONS OF THE RECTANGLES ARE RANDOM WITH AN         
;         EXPONENTIAL DISTRIBUTION. 16 CHECKERBOARDS ARE DISPLAYED IN         
;         TEST 1.                                                             
;            TEST 2 IS A MEMORY FUNCTION TEST. RANDOM BITS ARE STORED         
;         IN THE VM IN A SCRAMBLED ORDER WHICH IS ALSO RANDOMLY               
;         DETERMINED. AFTER EVERY MEMORY LOCATION IS FILLED, THE SAME         
;         DATA AND SEQUENCE IS REGENERATED AND MEMORY CONTENTS ARE            
;         CHECKED AGAINST IT. THEN A NEW SEQUENCE IS TRIED. THIS IS           
;         ITERATED 16 TIMES WITH A SEVERAL SECOND PAUSE BETWEEN THE WRITE     
;         AND VERIFY PHASE OF THE 16TH ITERATION INSERTED TO VERIFY THE       
;         FUNCTIONALITY OF DYNAMIC RAM REFRESH.                               
;            THIS PROGRAM IS SPECIFICALLY WRITTEN TO TEST 8K OF               
;         CONTIGUOUS MEMORY. MOOIFICATION TO TEST OTHER SIZES IS POSSIBLE     
;         BUT THE AMOUNT TESTED MUST BE A POWER OF 2.                         
;                                                                             
;                                                                             
;         KIM SYSTEM EQUATES                                                  
;                                                                             
KIMMON =        $1C22       ; ADDRESS OF SAVE MACHINE STATE ENTRY POINT     
VMORG  =        $2000       ; ADDRESS OF VISABLE MEMORY                     
VMSIZ  =        8192        ; SIZE OF VISABLE MEMORY BOARD                  
VMSG8T =        $1F         ;SIGNIFICANT UPPER ADDRESS BITS FOR VM          
;                                                                             
;         BASE PAGE DATA STORAGE                                              
         .org   0                                                            
;                                                                             
;         MAIN PROGRAM DATA STORAGE                                           
;                                                                             
ERRADR:  .WORD  0            ; ADDRESS OF DETECTED MEMORY ERROR              
ERRBTS:  .BYTE  0            ; ONES REPRESENT ERROR BITS                     
                                                                             
T1ITCT:  .BYTE  0            ; ITERATION COUNT FOR TEST 1                    
T2ITCT:  .BYTE  0            ; ITERATION COUNT FOR TEST 2                    
;                                                                             
;         DATA STORAGE FOR RANDOM PATTERN TEST                                
;                                                                             
RANDNO:  .WORD  1234         ; RANDOM MEMBER REGISTER                        
SEED:    .WORD  0            ; SAVES SEED FOR VERIFY                         
ADDRCT:  .WORD  0            ; DOUBLE BYTE ADDRESS COUNTER                   
SCHEMA:  .WORD  0            ; SCRAMBLED MEMORY ADDRESS AND ERROR ADDRES     
;                                                                             
;         DATA STORAGE FOR CHECKERBOARD TEST                                  
;                                                                             
VMADR:   .WORD  0            ; ADDRESS POINTER FOR VM DATA MANIPULATION      
VMDATA:  .BYTE  0            ; DATA DESTINED FOR VM                          
CKXSZ:   .BYTE  0            ; X SIZE (WIDTH) OF CHECKER RECTANGLE           
CKYSZ:   .BYTE  0            ; YSIZE (HEIGHT) OF CHECKER RECTANGLE 
CKDTA:   .BYTE  0            ; COLOR OF UPPER LEFT CHECKER RECTANGLE    
CKDTAX:  .BYTE  0            ; WORK COLOR DURING HORIZONTAL SCAN        
CKDTAY:  .BYTE  0            ; WORK COLOR DURING VERTICAL SCAN           
CKYCT:   .BYTE  0            ; COUNT OF CHECKER HEIGHT DURING VERTICAL  
                             ; SCAN                                     
HBYTCT:  .BYTE  0            ; BYTE COUNT DURING HORIZONTAL SCAN
; 		  MAIN TEST PROGRAM
          .ORG  $0200	     ; START PROGRAM CODE AT 200      
  
MTEST:   LDA    #$E0         ; INITIALIZE STACK POINTER                 
         TXS                                                             
         CLD                 ; INSURE BINARY ARITHMETIC                 
                                                                         
;         TEST 1  16 CHECKERBOARD PATTERNS                                
                                                                         
MAIN:    LDA    #16          ; SET 16 ITERATION COUNT                   
         STA    T1ITCT                                                   
MAIN1:   JSR    RNDEXP       ; GET AN EXPONENTIALLY DISTRIBUTED RANDOM  
                             ; NUMBER IN A                              
         STA    CKXSZ        ; MAKE IT THE X CHECKER SIZE               
         JSR    RNDEXP       ; GET ANOTHER                              
         STA    CKYSZ        ; MAKE IT THE Y CHECKER SIZE               
         JSR    RAND         ; RANDOMLY DETERMINE UPPER LEFT SQUARE     
         LDA    RANDNO       ; COLOR                                    
         ASL	A                                                           
         LDA    #0           ; SET A TO ALL ZEROES OR ALL ONES ACCORDING
         SBC    #0           ; TO SIGN OF RANDOM NUMBER                 
         STA    CKDTA                                                    
         JSR    CKGEN        ; GENERATE A CHECKERBOARD                  
         JSR    CKVER        ; VERIFY IT                                
         BNE    CKERLG       ; GO TO ERROR LOG IF ERROR                 
         DEC    T1ITCT       ; DECREMENT AND CHECK 16 ITERATION COUNTER 
         BNE    MAIN1        ; LOOP UNTIL 16 ITERATIONS DONE            
                                                                         
;        TEST 2 16 PASSES WITH RANDOM DATA    PAUSE IN 16TH PASS         
                                                                         
MAINIO:  LDA    #15          ; SET 16 ITERATION COUNT                   
         STA    T2ITCT                                                   
MAIN11:  JSR    RAND         ; NEW PASS, GET A RANDOM                   
         LDA    RANDNO       ; NUMBER IN RANDNO AND SAVE                
         STA    SEED         ; AS SEED FOR VERIFY                       
         LDA    RANDNO+1                                                 
         STA    SEED+1                                                   
         JSR    RNDGEN       ; GENERATE A RANDOM DATA PATTERN IN VM     
         LDA    T2ITCT       ; TEST IF LAST PASS                        
         BNE    MAIN15       ; SKIP OVER WAIT IF NOT                    
         LDX    #0           ; WAIT FOR ABOUT 5 SECONDS IN A TIGHT LOOP 
MAIN12:  LDY    #0                                                       
MAIN13:  LDA    #16                                                      
MAIN14:  CLC                                                             
         ADC    #-1                                                      
         BNE    MAIN14                                                   
         DEY                                                             
         BNE    MAIN13                                                   
         DEX                                                             
         BNE    MAIN12                                                   
MAIN15:  LDA    SEED         ; RESTORE RANDOM SEED FOR VERIFY PHASE     
         STA    RANDNO                                                   
         LDA    SEED+1                                                   
         STA    RANDNO+1                                                 
         JSR    RNOVER       ; VERIFY     
         BNE    RNERLG       ; GO TO ERROR LOG IF ERROR              
         DEC    T2ITCT       ; DECREMENT AND CHECK ITERATION COUNT   
         BPL    MAIN11       ; LOOP UNTIL 16 ITERATIONS DONE         
         JMP    MAIN         ; REPEAT THE ENTIRE TEST WITH DIFFERENT 
                             ; DATA                                  
                                                                       
;         ERROR LOG ROUTINES                                           
                                                                       
CKERLG:   STA    ERRBTS      ; STORE ERROR BITS                      
          LDA    VMADR       ; STORE ERROR ADDRESS                   
          STA    ERRADR                                                
          LDA    VMADR+1                                               
          STA    ERRADR+1                                              
          JMP    KIMMON      ; GO TO KIM MONITOR                     
                                                                       
RNERLG:   STA    ERRBTS      ; STORE ERROR BITS                      
          LDA    SCHEMA      ; STORE ERROR ADDRESS                   
          STA    ERRADR                                                
          LDA    SCHEMA+1                                              
          STA    ERRADR+1                                              
          JMP    KIMMON      ; GO TO KIM MONITOR                     
;
;         .PAGE 'CHECKERBOARD PATTERN GENERATE AND VERIFY ROUTINES'   
;         CHECKERBOARD PATTERN GENERATOR                              
;         STARTS AT UPPER LEFT CORNER OF SCREEN AND GENERATES A CHECKE
;         BOARD PATTERN.                                              
;         ENTER WITH CKXSZ SET TO CHECKER SQUARE WIDTH AND CKYSZ SET T
;         CHECKER SQUARE HEIGHT AND CKDTA SET TO 0 FOR A BLACK UPPER L
;         SQUARE OR SET TO $FF FOR A WHITE UPPER LEFT SQUARE.        
;         USES ALL REGISTERS, PRESERVES CKXSZ, CKYSZ, CKDTA           
;                                                                     
CKGEN:   LDA   #VMORG/256   ; INITIALIZE ADDRESS POINTER TO BEGINNING
         STA   VMADR+1      ; OF VM                                  
         LDA   #0                                                    
         STA   VMADR                                                 
         LDY   #8           ; INITIALIZE BIT COUNT                   
         LDA   CKDTA        ; COPY CKDTA TO VERTICAL WORK LOCATION   
         STA   CKDTAY                                                
;                                                                     
;        START A ROW OF CHECKER BLOCKS                                
;                                                                     
CKGNV:   LDA   CKYSZ        ; SET Y SIZE IN CKYCT                    
         STA   CKYCT                                                 
                                                                     
;        STARTA HORIZONTAL SCAN                                      
                                                                     
CKGNH:   LDA   CKDTAY       ; COPY VERTICAL CKDTA TO HORIZONTAL WORK  
         STA   CKDTAX       ; LOCATION                               
         LDA   #40          ; INITIALIZE COUNT OF BYTES GENERATED IN 
         STA   HBYTCT       ; A HORIZONTAL SCAN                      
                                                                     
CKGNH1:  LDX   CKXSZ        ; SET X SIZE IN INDEX X                  
CKGNH2:  LDA   CKDTAX       ; GENERATE A DOT = TO CURRENT VALUE OF   
         ROL   A            ; CKDTAX                                  
         ROL   VMDATA                                                
         DEY                ; COUNT DOTS GENERATED                   
         BNE   CKGNH4       ; SKIP AHEAD IF NOT 8 YET                
         LDA   VMDATA       ; STORE A COMPLETED BYTE IN VM           
         STA   (VMADR),Y                                             
         INC   VMADR        ; INCREMENT VM ADDRESS                  
         BNE   CKGNH3                                                
         INC   VMADR+1                                               
         LDA   VMADR+1      ; TEST IF ENTIRE VM FILLED               
         CMP   #VMORG+8192/256                                       
         BEQ   CKGENF       ; JUMP OUT IF SO                         
CKGNH3:  LDY   #8           ; RESTORE 8 BIT COUNT                    
         DEC   HBYTCT       ; TEST IF FINISHED WITH A HORIZONTAL SCAN 
         BEQ   CKGNV1       ; JUMP IF SO                             
CKGNH4: DEX                 ; DECREMENT SQUARE WIDTH COUNT           
         BNE   CKGNH2       ; GO GENERATE NEXT DOT                   
         LDA   CKDTAX       ; AT SQUARE BOUNDARY, FLIP COLOR         
         EOR   #$FF                                                 
         STA   CKDTAX                                                
         JMP   CKGNH1       ; GO GENERATE NEXT DOT                   
;                                                                     
;        FINISH VERTICAL SCAN                                        
;                                                                        
CKGNV1: DEC     CKYCT       ; DECREMENT SQUARE HEIGHT                   
         BNE    CKGNH       ; GO GENERATE NEXT LINE                     
         LDA    CKDTAY      ; AT SQUARE BOUNDARY, FLIP COLOR            
         EOR    #$FF                                                   
         STA    CKDTAY                                                  
         JMP    CKGNV       ; GO GENERATE NEXT LINE                     
                                                                        
CKGENF:  RTS                ; RETURN                                    
                                                                        
;         CHECKERBOARD PATTERN VERIFY                                    
;         STARTS AT UPPER LEFT CORNER OF SCREEN AND VERIFIES A CHECKER-  
;         BOARD PATTERN.                                                 
;         ENTER WITH CKXSZ SET TO CHECKER SQUARE WIDTH AND CKYSZ SET TO  
;         CHECKER SQUARE HEIGHT AND CKDTA SET TO 0 FOR A BLACK UPPER LEFT
;         SQUARE OR SET TO $FF FOR A WHITE UPPER LEFT SQUARE.           
;         USES ALL REGISTERS, PRESERVES CKXSZ, CKYSZ, CKDTA              
;         UPON FINDING A MISMATCH BETWEEN THE GENERATED PATTERN AND THE  
;         CONTENTS OF THE VM, RETURNS WITH THE ERROR BIT SET IN A AND    
;         THE ADDRESS OF THE ERROR IN VMADR AND VMADR+1                  
;                                                                        
CKVER:   LDA    #VMORG/256   ; INITIALIZE ADDRESS POINTER TO BEGINNING  
         STA    VMADR+1      ; OF VM                                    
         LDA    #0                                                      
         STA    VMADR                                                   
         LDY    #8           ; INITIALIZE BIT COUNT                     
         LDA    CKDTA        ; COPY CKDTA TO VERTICAL WORK LOCATION     
         STA    CKDTAY                                                  
;                                                                        
;         START A ROW OF CHECKER BLOCKS                                  
;                                                                        
CKVRV:   LDA    CKYSZ        ; SET Y SIZE IN CKYCT                      
         STA    CKYCT                                                   
;                                                                        
;         START A HORIZONTAL SCAN                                        
;                                                                        
CKVRH:   LDA     CKDTAY      ; COPY VERTICAL CKDTA TO HORIZONTAL WORK   
         STA     CKDTAX      ; LOCATION                                 
         LDA     #40         ; INITIALIZE COUNT OF BYTES GENERATED IN   
         STA     HBYTCT      ; A HORIZONTAL SCAN                        
CKVRH1:  LDX     CKXSZ       ; SET X SIZE IN INDEX X                    
CKVRH2:  LDA     CKDTAX      ; GENERATE A DOT = TO CURRENT VALUE OF     
         ROL     A           ; CKDTAX                                   
         ROL     VMDATA                                                 
         DEY                 ; COUNT DOTS GENERATED                     
         BNE     CKVRH4      ; SKIP AHEAD IF NOT 8 YET                  
         LDA     VMDATA      ; COMPARE THE COMPLETED BYTE WITH VM       
         EOR     (VMADR),Y   ; CONTENTS                                 
         BNE     CKVERF      ; JUMP IF ERROR                            
         INC     VMADR       ; INCREMENT VM ADDRESS                     
         BNE     CKVRH3                                                 
         INC     VMADR+1                                                
         LDA     VMADR+1     ; TEST IF ENTIRE VM FILLED                 
         EOR     #VMORG+8192/256                                        
         BEQ     CKVERF      ; JUMP OUT IF SO                           
CKVRH3:  LDY     #8          ; RESTORE 8 BIT COUNT                    
         DEC     HBYTCT      ; TEST IF FINISHED WITH A HORIZONTAL SCAN
         BEQ     CKVRV1      ; JUMP IF SO                             
CKVRH4:  DEX                 ; DECREMENT SQUARE WIDTH COUNT           
         BNE     CKVRH2      ; GO GENERATE NEXT DOT                   
         LDA     CKDTAX      ; AT SQUARE BOUNDARY, FLIP COLOR         
         EOR     #$FF                                              
         STA     CKDTAX                                             
         JMP     CKVRH1      ; GO GENERATE NEXT DOT                   
;                                                                
;        FINISH VERTICAL SCAN                                    
;                                                                
CKVRV1:  DEC     CKYCT       ; DECREMENT SQUARE HEIGHT                
         BNE     CKVRH       ; GO GENERATE NEXT LINE                  
         LDA     CKDTAY      ; AT SQUARE BOUNDARY, FLIP COLOR         
         EOR     #$FF        ;                                       
         STA     CKDTAY                                             
         JMP     CKVRV       ; GO GENERATE NEXT LINE                  
                                                                
CKVERF:  RTS                 ; RETURN                                 
;        .PAGE 'RANDOM PATTERN GENERATE AND VERIFY ROUTINES'        
;        RANDOM PATTERN STORED IN SCRAMBLED ORDER GENERATE ROUTINE  
                                                                  
RNDGEN:  LDA     #0          ; INITIALIZE ADDRESS COUNTER              
         STA     ADDRCT      ; TO 8192                                 
         LDA     #VMSIZ/256                                           
         STA     ADDRCT+1                                             
STORPH:  JSR     RAND        ; GENERATE A RANDOM NUMBER                 
         JSR     MADDR       ; FORM A SCRAMBLED MEMORY ADDRESS          
         LDA     RANDNO      ; STORE A RANDOM BYTE                      
         LDX     #0          ; INDIRECTLY THROUGH SCRAMBLED MEMORY      
         STA     (SCHEMA,X)  ; ADDRESS AT SCHEMA                        
		 DEC     ADDRCT      ; DECREMENT ADDRESS COUNTER                
		 BNE     STORPH       ; AND LOOP IF NOT ZERO                     
		 DEC     ADDRCT+1                                             
		 BNE     STORPH                                               
  	     RTS                 ;  RETURN WHEN DONE                         
;                                                                 
;                                                                  
;       RANDOM PATTERN STORED IN SCRAMBLED ORDER VERIFY ROUTINE    
;                                                                  
RNOVER:  LDA     #VMSIZ/256  ; INITIALIZE ADDRESS COUNTER              
         STA     ADDRCT+1                                             
VERFPH:  JSR     RAND        ; GENERATE A RANDOM NUMBER                
         JSR     MADDR       ; FORM SCRAMBLED MEMORY ADDRESS           
         LDA     (SCHEMA,X)  ; GET DATA FROM MEMORY INDIRECTLY         
         EOR     RANDNO      ; THROUGH SCMEMA                          
         BNE     VERRET      ; GO RETURN ON UNEQUAL COMPARE            
         DEC     ADDRCT      ; DECREMENT ADDRESS COUNTER               
         BNE     VERFPH      ; AND LOOP IF NOT ZERO                    
         DEC     ADDRCT+1                                             
         BNE     VERFPH                                               
VERRET:  RTS                 ; RETURN                                  
;       SCRAMBLED MEMORY ADDRESS FORMATION ROUTINE                 
;       USES ADDRCT AND SEED TO FORM A SCRAMBLED ADDRESS IN SCHEMA 
;                                                                  
MADDR:   LDA     SEED        ; GET LOWER BYTE OF RANDOM NUMBER          
         EOR     ADDRCT      ; EXCLUSIVE-OR WITH LOWER ADDRESS          
         STA     SCHEMA      ; LOWER BYTE OF RESULT                     
         LDA     SEED+1      ; GET UPPER BYTE OF RANDOM NUMBER          
         EOR     ADDRCT+1    ; EXCLUSIVE-OR WITH UPPER ADDRESS          
         AND     #VMSG8T     ; SAVE SIGNIFICANT BITS OF RESULT          
         CLC                                                        
         ADC     #VMORG/256  ;ADD IN FIRST PAGE NUMBER OF BOARD        
         STA     SCHEMA+1    ; BEING TESTED                             
         RTS                 ; RETURN                                   
;                                                                  
;                                                                  
;       RANDOM NUMBER GENERATOR SUBROUTINE                         
;       ENTER WITH SEED IN RANDNO                                  
;       EXIT WITH NEW RANDOM NUMBER IN RANDNO                      
;       USES 16 BIT FEEDBACK SHIFT REGISTER METHOD                 
;       DESTROYS REGISTER A AND Y                                  
;                                                                  
RAND:    LDY     #8          ; SET COUNTER FOR 8 RANDOM BITS           
RAND1:   LDA     RANDNO      ; EXCLUSIVE-OR BITS 3, 12, 14, AND 15       
         LSR     A           ;  OF SEED                                   
         EOR     RANDNO                                                 
         LSR     A                                                         
         LSR     A                                                         
         EOR     RANDNO                                                 
         LSR     A                                                         
         EOR     RANDNO+1    ; RESULT IS IN BIT 3 OF A                   
         LSR     A           ; SHIFT INTO CARRY                          
         LSR     A                                                         
         LSR     A                                                         
         LSR     A                                                         
         ROL     RANDNO+1    ; SHIFT RANDNO LEFT ONE BRINGING IN CARRY   
         ROL     RANDNO                                                 
         DEY                 ; TEST IF 8 NEW RANDOM BITS COMPUTED        
         BNE     RAND1       ; LOOP FOR MORE IF NOT                      
         RTS                 ; RETURN                                    
;                                                                    
;      EXPONENTIALLY DISTRIBUTED RANDOM NUMBER SUBROUTINE           
;      RULES OF USE SAME AS RAND, 8 BIT RESULT RETURNED IN A        
;      AN EXPONENTIAL DISTRIBUTION MEANS THAT THE PROBABILITY OF A  
;      RESULT BETWEEN 10 AND 20 IS THE SAME AS THE PROBABILITY OF A 
;      RESULT BETWEEN 100 AND 200.                                  
;      NOTE THAT THE PROBABILITY OF A ZERO RESULT IS ZERO.          
;                                                                    
RNDEXP:  JSR     RAND        ; GET TWO NEW RANDOM BYTES                  
         JSR     RAND                                                   
         LDA     RANDNO      ; CONVERT ONE OF THE BYTES TO A RANDOM      
         AND     #7          ; VALUE BETWEEN 0 AND 7 AND PUT IN Y AS A   
         TAY                 ; SHIFT COUNT                               
         INY                                                          
         LDA     RANDNO+1    ; GET THE OTHER RANDOM NUMBER AND SHIFT IT  
RNDXPl:  DEY                 ; RIGHT ACCORDING TO Y                      
         BEQ     RNDXP2                                                 
         LSR     A                                                         
         JMP     RNDXPl                                                 
RNDXP2:  ORA     #0          ; TEST FOR A ZERO RESULT                    
         BEQ     RNDEXP      ; PROHIBIT ZERO RESULTS                     
         RTS                 ; RETURN                                    
                                                                    
        .END